@@ -25,6 +25,7 @@
 #include <libsoup/soup-gnome.h>
 #include <libsoup/soup-message.h>
 #include "couchdb-session.h"
+#include "couchdb-document.h"
 #include "couchdb-document-info.h"
 #include "couchdb-marshal.h"
 #include "dbwatch.h"
@@ -32,6 +33,7 @@
 #include <string.h>
 #ifdef HAVE_OAUTH
 #include <time.h>
+#include <stdlib.h>
 #include "oauth.h"
 #endif
 
@@ -109,7 +111,7 @@ couchdb_session_set_property (GObject         *object,
 	switch (prop_id) {
 	case PROP_URI:
 		g_free(couchdb->priv->uri);
-		couchdb->priv->uri = g_value_dup_string(value);
+		couchdb->priv->uri = g_value_dup_string (value);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -234,12 +236,32 @@ couchdb_session_init (CouchdbSession *couchdb)
 #endif
 }
 
+/**
+ * couchdb_session_new:
+ * @uri: URI of the CouchDB instance to connect to
+ *
+ * Create a new #CouchdbSession object, which is the entry point for operations on a
+ * CouchDB instance.
+ *
+ * Return value: A newly-created #CouchdbSession object.
+ */
 CouchdbSession *
 couchdb_session_new (const char *uri)
 {
+	if (!uri)
+		uri = "http://127.0.0.1:5984";
+
 	return g_object_new (COUCHDB_TYPE_SESSION, "uri", uri, NULL);
 }
 
+/**
+ * couchdb_session_get_uri:
+ * @couchdb: A #CouchdbSession object
+ *
+ * Retrieve the URI of the CouchDB instance a #CouchdbSession object is bound to.
+ *
+ * Return value: the URI of the CouchDB instance used by this #CouchdbSession object.
+ */
 const char *
 couchdb_session_get_uri (CouchdbSession *couchdb)
 {
@@ -248,6 +270,17 @@ couchdb_session_get_uri (CouchdbSession *couchdb)
 	return (const char *) couchdb->priv->uri;
 }
 
+/**
+ * couchdb_session_list_databases:
+ * @couchdb: A #CouchdbSession object
+ * @error: Placeholder for error information
+ *
+ * Retrieve the list of databases that exist in the CouchDB instance being used.
+ *
+ * Return value: A list of strings containing the names of all the databases
+ * that exist in the CouchDB instance connected to. Once no longer needed, this
+ * list can be freed by calling #couchdb_session_free_database_list.
+ */
 GSList *
 couchdb_session_list_databases (CouchdbSession *couchdb, GError **error)
 {
@@ -275,14 +308,25 @@ couchdb_session_list_databases (CouchdbSession *couchdb, GError **error)
 			}
 		}		
 	}
-	g_object_unref (G_OBJECT (parser));
 
 	/* Free memory */
+	g_object_unref (G_OBJECT (parser));
 	g_free (url);
 
 	return dblist;
 }
 
+/**
+ * couchdb_session_get_database_info:
+ * @couchdb: A #CouchdbSession object
+ * @dbname: Name of the database for which to retrieve the information
+ * @error: Placeholder for error information
+ *
+ * Retrieve information about a given database.
+ *
+ * Return value: A #CouchdbDatabaseInfo object, whose API can be used to retrieve
+ * all the information returned by CouchDB about this database.
+ */
 CouchdbDatabaseInfo *
 couchdb_session_get_database_info (CouchdbSession *couchdb, const char *dbname, GError **error)
 {
@@ -315,6 +359,16 @@ couchdb_session_get_database_info (CouchdbSession *couchdb, const char *dbname,
 	return result;
 }
 
+/**
+ * couchdb_session_create_database
+ * @couchdb: A #CouchdbSession object
+ * @dbname: Name of the database to be created
+ * @error: Placeholder for error information
+ *
+ * Create a new database on a CouchDB instance.
+ *
+ * Return value: TRUE if successful, FALSE otherwise.
+ */
 gboolean
 couchdb_session_create_database (CouchdbSession *couchdb, const char *dbname, GError **error)
 {
@@ -345,6 +399,16 @@ couchdb_session_create_database (CouchdbSession *couchdb, const char *dbname, GE
 	return result;
 }
 
+/**
+ * couchdb_session_delete_database
+ * @couchdb: A #CouchdbSession object
+ * @dbname: Name of the database to be deleted
+ * @error: Placeholder for error information
+ *
+ * Delete an existing database on a CouchDB instance.
+ *
+ * Return value: TRUE if successful, FALSE otherwise.
+ */
 gboolean
 couchdb_session_delete_database (CouchdbSession *couchdb, const char *dbname, GError **error)
 {
@@ -365,8 +429,8 @@ couchdb_session_delete_database (CouchdbSession *couchdb, const char *dbname, GE
 			result = json_object_get_boolean_member (
 				json_node_get_object (root_node), "ok");		
 	}
-	g_object_unref (G_OBJECT (parser));
 
+	g_object_unref (G_OBJECT (parser));
 	g_free (url);
 
 	if (result) {
@@ -380,6 +444,51 @@ couchdb_session_delete_database (CouchdbSession *couchdb, const char *dbname, GE
 	return result;
 }
 
+/**
+ * couchdb_session_compact_database:
+ * @couchdb: A #CouchdbSession object
+ * @dbname: Name of the database to be compacted
+ * @error: Placeholder for error information
+ *
+ * Compact the given database, which means removing outdated document
+ * revisions and deleted documents.
+ *
+ * Return value: TRUE if successful, FALSE otherwise.
+ */
+gboolean
+couchdb_session_compact_database (CouchdbSession *couchdb, const char *dbname, GError **error)
+{
+	char *url;
+	JsonParser *output;
+	gboolean result = FALSE;
+
+	g_return_val_if_fail (COUCHDB_IS_SESSION (couchdb), FALSE);
+	g_return_val_if_fail (dbname != NULL, FALSE);
+
+	url = g_strdup_printf ("%s/%s/_compact", couchdb_session_get_uri (couchdb), dbname);
+	output = json_parser_new ();
+
+	if (couchdb_session_send_message (couchdb, SOUP_METHOD_POST, url, NULL, output, error)) {
+		JsonNode *root_node;
+
+		root_node = json_parser_get_root (output);
+		if (json_node_get_node_type (root_node) == JSON_NODE_OBJECT)
+			result = json_object_get_boolean_member (
+				json_node_get_object (root_node), "ok");
+	}
+
+	g_object_unref (G_OBJECT (output));
+	g_free (url);
+
+	return result;
+}
+
+/**
+ * couchdb_session_free_database_list:
+ * @dblist: A list of databases, as returned by #couchdb_session_list_databases
+ *
+ * Free the list of databases returned by #couchdb_session_list_databases.
+ */
 void
 couchdb_session_free_database_list (GSList *dblist)
 {
@@ -391,8 +500,18 @@ couchdb_session_free_database_list (GSList *dblist)
 
 /**
  * couchdb_session_list_documents:
+ * @couchdb: A #CouchdbSession object
+ * @dbname: Name of the databases to retrieve documents from
+ * @error: Placeholder for error information
  *
- * Return Value: (element-type Couchdb.DocumentInfo) (transfer full)
+ * Retrieve the list of all documents from a database on a running CouchDB instance.
+ * For each document, a #CouchdbDocumentInfo object is returned on the list, which
+ * can then be used for retrieving specific information for each document.
+ *
+ * Return Value: a list of #CouchdbDocumentInfo objects, or NULL if there are none
+ * or there was an error (in which case the error argument will contain information
+ * about the error). Once no longer needed, the list should be freed by calling
+ * #couchdb_session_free_document_list.
  */
 GSList *
 couchdb_session_list_documents (CouchdbSession *couchdb, const char *dbname, GError **error)
@@ -433,12 +552,20 @@ couchdb_session_list_documents (CouchdbSession *couchdb, const char *dbname, GEr
 			}
 		}
 	}
+
 	g_object_unref (G_OBJECT (parser));
 	g_free (url);
 
 	return doclist;
 }
 
+/**
+ * couchdb_session_free_document_list:
+ * @doclist: A list of #CouchdbDocumentInfo objects, as returned by
+ * #couchdb_session_list_documents
+ *
+ * Free the list of documents returned by #couchdb_session_list_documents.
+ */
 void
 couchdb_session_free_document_list (GSList *doclist)
 {
@@ -448,6 +575,20 @@ couchdb_session_free_document_list (GSList *doclist)
 	g_slist_free (doclist);
 }
 
+/**
+ * couchdb_session_listen_for_changes:
+ * @couchdb: A #CouchdbSession object
+ * @dbname: Name of the database to poll changes for
+ *
+ * Setup a listener to get information about changes done to a specific database. Please
+ * note that changes done in the application using couchdb-glib will be notified
+ * without the need of calling this function. But if the application wants to receive
+ * notifications of changes done externally (by another application, or by any other
+ * means, like replication with a remote database), it needs to call this function.
+ *
+ * For each change, one of the signals on the #CouchdbSession object will be emitted,
+ * so applications just have to connect to those signals before calling this function.
+ */
 void
 couchdb_session_listen_for_changes (CouchdbSession *couchdb, const char *dbname)
 {
@@ -489,7 +630,8 @@ couchdb_session_listen_for_changes (CouchdbSession *couchdb, const char *dbname)
  * @couchdb: A #CouchdbSession object
  * @credentials: A #CouchdbCredentials object
  *
- * Enables authentication for the given #CouchdbSession object.
+ * Enables authentication for the given #CouchdbSession object. The authentication
+ * mechanism should be specificied when creating the #CouchdbCredentials object.
  */
 void
 couchdb_session_enable_authentication (CouchdbSession *couchdb,
@@ -679,16 +821,36 @@ parse_json_response (CouchdbSession *couchdb, JsonParser *json_parser, SoupMessa
 	return success;
 }
 
+/**
+ * couchdb_session_send_message:
+ * @couchdb: A #CouchdbSession object
+ * @method: HTTP method to use
+ * @url: URL to send the message to
+ * @body: Body of the HTTP request
+ * @output: Placeholder for output information
+ * @error: Placeholder for error information
+ *
+ * This function is used to communicate with CouchDB over HTTP, and should not be used
+ * by applications unless they really have a need (like missing API in couchdb-glib which
+ * the application needs).
+ *
+ * Return value: TRUE if successful, FALSE otherwise.
+ */
 gboolean
-couchdb_session_send_message (CouchdbSession *couchdb, const char *method, const char *url, const char *body, JsonParser *parser, GError **error)
+couchdb_session_send_message (CouchdbSession *couchdb, const char *method, const char *url, const char *body, JsonParser *output, GError **error)
 {
 	SoupMessage *http_message;
 	guint status;
+	GError **real_error;
 	
 	g_return_val_if_fail (COUCHDB_IS_SESSION (couchdb), FALSE);
 	g_return_val_if_fail (method != NULL, FALSE);
-	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
-	
+
+	if (error != NULL)
+		real_error = error;
+	else
+		*real_error = NULL;
+
 	http_message = soup_message_new (method, url);
 	if (body != NULL) {
 		soup_message_set_request (http_message, "application/json", SOUP_MEMORY_COPY,
@@ -713,11 +875,12 @@ couchdb_session_send_message (CouchdbSession *couchdb, const char *method, const
 #endif
 	status = soup_session_send_message (couchdb->priv->http_session, http_message);
 	if (SOUP_STATUS_IS_SUCCESSFUL (status)) {
-		if (parser != NULL)
-		       	parse_json_response (couchdb, parser, http_message, error);
+		if (output != NULL)
+		       	parse_json_response (couchdb, output, http_message, real_error);
 	       	return TRUE;
 	} else {
-		g_set_error (error, COUCHDB_ERROR, status, "%s", http_message->reason_phrase);
+		if (error != NULL)
+			g_set_error (error, COUCHDB_ERROR, status, "%s", http_message->reason_phrase);
 		return FALSE;
 	}
 }
@@ -733,7 +896,7 @@ static void
 debug_message (const gchar *log_domain, GLogLevelFlags log_level,
 	       const gchar *message, gpointer user_data)
 {
-	gchar* couchdb_env_debug_messages;
+	const gchar* couchdb_env_debug_messages;
 
 	couchdb_env_debug_messages = g_getenv(COUCHDB_ENV_DEBUG_MESSAGES);
 	if (couchdb_env_debug_messages != NULL) {
@@ -742,3 +905,60 @@ debug_message (const gchar *log_domain, GLogLevelFlags log_level,
 }
 #endif
 
+/**
+ * couchdb_session_replicate:
+ * @couchdb: A #CouchdbSession object
+ * @source: Source database
+ * @target: Target database
+ * @continous: Whether to replicate once or keep replicating
+ * @error: Placeholder for error information
+ *
+ * Replicates a source database to another database, on the same CouchDB instance
+ * or on a remote instance.
+ *
+ * If @continous is FALSE, the replication will happen once, but if set to TRUE,
+ * CouchDB will listen to all changes made to the source database, and automatically
+ * replicate over any new docs as the come into the source to the target.
+ *
+ * Return value: TRUE if successful, FALSE otherwise, in which case the @error
+ * parameter will be set to contain information about the error.
+ */
+gboolean
+couchdb_session_replicate (CouchdbSession *couchdb,
+			   const gchar *source,
+			   const gchar *target,
+			   gboolean continous,
+			   GError **error)
+{
+	char *url, *body;
+	CouchdbDocument *input;
+	JsonParser *output;
+	gboolean send_ok;
+
+	g_return_val_if_fail (COUCHDB_IS_SESSION (couchdb), FALSE);
+	g_return_val_if_fail (source != NULL, FALSE);
+	g_return_val_if_fail (target != NULL, FALSE);
+
+	/* Build the input document */
+	input = couchdb_document_new (couchdb);
+	couchdb_document_set_string_field (input, "source", source);
+	couchdb_document_set_string_field (input, "target", target);
+	if (continous)
+		couchdb_document_set_boolean_field (input, "continous", TRUE);
+
+	/* Send message */
+	url = g_strdup_printf ("%s/_replicate", couchdb_session_get_uri (couchdb));
+	body = couchdb_document_to_string (input);
+	output = json_parser_new ();
+
+	send_ok = couchdb_session_send_message (couchdb, SOUP_METHOD_POST, url, body, output, error);
+	/* FIXME: what to do with the information returned? -> http://books.couchdb.org/relax/reference/replication */
+
+	/* Free memory */
+	g_object_unref (G_OBJECT (output));
+	g_object_unref (G_OBJECT (input));
+	g_free (body);
+	g_free (url);
+
+	return send_ok;
+}
