@@ -37,4 +37,13 @@ GQuark      couchdb_error_quark (void);
 
 char* generate_uuid (void);
 
+/* Private API */
+JsonObject*	  couchdb_document_get_json_object	(CouchdbDocument *document);
+
+CouchdbArrayField  *couchdb_array_field_new_from_json_array (JsonArray *json_array);
+JsonArray          *couchdb_array_field_get_json_array (CouchdbArrayField *array);
+
+CouchdbStructField *couchdb_struct_field_new_from_json_object (JsonObject *json_object);
+JsonObject         *couchdb_struct_field_get_json_object (CouchdbStructField *sf);
+
 #endif
