@@ -45,6 +45,25 @@ desktopcouch_document_is_contact (CouchdbDocument *document)
 				    DESKTOPCOUCH_RECORD_TYPE_CONTACT);
 }
 
+const char *
+desktopcouch_document_contact_get_title (CouchdbDocument *document)
+{
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
+	g_return_val_if_fail (desktopcouch_document_is_contact (document), NULL);
+
+	return couchdb_document_get_string_field (document, "title");
+}
+
+void
+desktopcouch_document_contact_set_title (CouchdbDocument *document, const char *title)
+{
+      	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
+	g_return_if_fail (desktopcouch_document_is_contact (document));
+	g_return_if_fail (title != NULL);
+
+	couchdb_document_set_string_field (document, "title", title);
+}
+
 const char *
 desktopcouch_document_contact_get_first_name (CouchdbDocument *document)
 {
@@ -64,6 +83,25 @@ desktopcouch_document_contact_set_first_name (CouchdbDocument *document, const c
 	couchdb_document_set_string_field (document, "first_name", first_name);
 }
 
+const char *
+desktopcouch_document_contact_get_middle_name (CouchdbDocument *document)
+{
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
+	g_return_val_if_fail (desktopcouch_document_is_contact (document), NULL);
+
+	return couchdb_document_get_string_field (document, "middle_name");
+}
+
+void
+desktopcouch_document_contact_set_middle_name (CouchdbDocument *document, const char *middle_name)
+{
+	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
+	g_return_if_fail (desktopcouch_document_is_contact (document));
+	g_return_if_fail (middle_name != NULL);
+
+	couchdb_document_set_string_field (document, "middle_name", middle_name);
+}
+
 const char *
 desktopcouch_document_contact_get_last_name (CouchdbDocument *document)
 {
@@ -83,6 +121,25 @@ desktopcouch_document_contact_set_last_name (CouchdbDocument *document, const ch
 	couchdb_document_set_string_field (document, "last_name", last_name);
 }
 
+const char *
+desktopcouch_document_contact_get_suffix (CouchdbDocument *document)
+{
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
+	g_return_val_if_fail (desktopcouch_document_is_contact (document), NULL);
+
+	return couchdb_document_get_string_field (document, "suffix");
+}
+
+void
+desktopcouch_document_contact_set_suffix (CouchdbDocument *document, const char *suffix)
+{
+      	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
+	g_return_if_fail (desktopcouch_document_is_contact (document));
+	g_return_if_fail (suffix != NULL);
+
+	couchdb_document_set_string_field (document, "suffix", suffix);
+}
+
 const char *
 desktopcouch_document_contact_get_nick_name (CouchdbDocument *document)
 {
@@ -201,25 +258,6 @@ desktopcouch_document_contact_set_department (CouchdbDocument *document, const c
 	couchdb_document_set_string_field (document, "department", department);
 }
 
-const char *
-desktopcouch_document_contact_get_title (CouchdbDocument *document)
-{
-	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
-	g_return_val_if_fail (desktopcouch_document_is_contact (document), NULL);
-
-	return couchdb_document_get_string_field (document, "title");
-}
-
-void
-desktopcouch_document_contact_set_title (CouchdbDocument *document, const char *title)
-{
-      	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
-	g_return_if_fail (desktopcouch_document_is_contact (document));
-	g_return_if_fail (title != NULL);
-
-	couchdb_document_set_string_field (document, "title", title);
-}
-
 const char *
 desktopcouch_document_contact_get_job_title (CouchdbDocument *document)
 {
